package com.b510.big.data.flume.client;

import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.apache.flume.Event;
import org.apache.flume.EventDeliveryException;
import org.apache.flume.api.RpcClient;
import org.apache.flume.api.RpcClientFactory;
import org.apache.flume.event.EventBuilder;

/**
 * @author Hongten
 * 
 *         功能： 模拟产生用户日志信息，并且向Flume发送数据
 */
public class FlumeClient {

	public static void main(String[] args) {
		ExecutorService exec = Executors.newCachedThreadPool();
		exec.execute(new GenerateDataAndSend2Flume());

		exec.shutdown();
	}

}

class GenerateDataAndSend2Flume implements Runnable {

	FlumeRPCClient flumeRPCClient;
	static Random random = new Random();

	GenerateDataAndSend2Flume() {
		// 初始化RPC客户端
		flumeRPCClient = new FlumeRPCClient();
		flumeRPCClient.init(Common.FLUME_HOST_NAME, Common.FLUME_PORT);
	}

	@Override
	public void run() {
		while (true) {
			Date date = new Date();
			SimpleDateFormat simpleDateFormat = new SimpleDateFormat(Common.DATE_FORMAT_YYYYDDMM);
			String d = simpleDateFormat.format(date);
			Long timestamp = new Date().getTime();
			// ip地址生成
			String ip = random.nextInt(Common.MAX_IP_NUMBER) + "." + random.nextInt(Common.MAX_IP_NUMBER) + "." + random.nextInt(Common.MAX_IP_NUMBER) + "." + random.nextInt(Common.MAX_IP_NUMBER);
			// ip地址对应的address(这里是为了构造数据，并没有按照真实的ip地址，找到对应的address)
			String address = Common.ADDRESS[random.nextInt(Common.ADDRESS.length)];

			Long userid = Math.abs(random.nextLong());
			String action = Common.USER_ACTION[random.nextInt(Common.USER_ACTION.length)];
			// 日志信息构造
			// example : 199.80.45.117 云南 2018-12-20 1545285957720 3086250439781555145 www.hongten.com Buy
			String data = ip + "\t" + address + "\t" + d + "\t" + timestamp + "\t" + userid + "\t" + Common.WEB_SITE + "\t" + action;
			//System.out.println(data);

			// 往Flume发送数据
			flumeRPCClient.sendData2Flume(data);

			try {
				TimeUnit.MICROSECONDS.sleep(random.nextInt(1000));
			} catch (InterruptedException e) {
				flumeRPCClient.cleanUp();
				System.out.println("interrupted exception : " + e);
			}
		}
	}
}

class FlumeRPCClient {

	private RpcClient client;
	private String hostname;
	private int port;

	public void init(String hostname, int port) {
		this.hostname = hostname;
		this.port = port;
		this.client = getRpcClient(hostname, port);
	}

	public void sendData2Flume(String data) {
		Event event = EventBuilder.withBody(data, Charset.forName(Common.CHAR_FORMAT));

		try {
			client.append(event);
		} catch (EventDeliveryException e) {
			cleanUp();
			client = null;
			client = getRpcClient(hostname, port);
		}
	}

	public RpcClient getRpcClient(String hostname, int port) {
		return RpcClientFactory.getDefaultInstance(hostname, port);
	}

	public void cleanUp() {
		// Close the RPC connection
		client.close();
	}
}

// 所有的常量定义
class Common {
	public static final String CHAR_FORMAT = "UTF-8";

	public static final String DATE_FORMAT_YYYYDDMM = "yyyy-MM-dd";

	// this is a test web site
	public static final String WEB_SITE = "www.hongten.com";

	// 用户的action
	public static final String[] USER_ACTION = { "Register", "Login", "View", "Click", "Double_Click", "Buy", "Shopping_Car", "Add", "Edit", "Delete", "Comment", "Logout" };

	public static final int MAX_IP_NUMBER = 224;
	// ip所对应的地址
	public static String[] ADDRESS = { "北京", "天津", "上海", "广东", "重庆", "河北", "山东", "河南", "云南", "山西", "甘肃", "安徽", "福建", "黑龙江", "海南", "四川", "贵州", "宁夏", "新疆", "湖北", "湖南", "山西", "辽宁", "吉林", "江苏", "浙江", "青海", "江西", "西藏", "内蒙", "广西", "香港", "澳门", "台湾", };

	// Flume conf
	public static final String FLUME_HOST_NAME = "node2";
	public static final int FLUME_PORT = 41414;

}