package com.b510.big.data.poi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HConnection;
import org.apache.hadoop.hbase.client.HConnectionManager;
import org.apache.hadoop.hbase.client.HTableInterface;
import org.apache.hadoop.hbase.client.Result;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class ReportUtil {

	public static void main(String[] args) throws Exception {

		String year = "2018";
		String month = "12";
		String day = "21";
		String hour = "14";

		generateReport(year, month, day, hour);
	}

	private static void generateReport(String year, String month, String day, String hour) {
		HBaseDAO hBaseDAO = new HBaseDAOImpl();
		// format: yyyyMMddHH
		String begin = year + month + day + hour;
		String[] split = generateQuailifers(begin);

		List<Integer> userBuyCountList = getData(hBaseDAO, year, month, day, split, Common.KEY_WORD_BUY);
		List<Integer> userShoppingCarCountList = getData(hBaseDAO, year, month, day, split, Common.KEY_WORD_SHOPPING_CAR);

		//System.err.println(userBuyCountList.size());
		//System.err.println(userShoppingCarCountList.size());

		writeExcel(year, month, day, hour, userBuyCountList, userShoppingCarCountList);
	}

	private static void writeExcel(String year, String month, String day, String hour, List<Integer> userBuyCountList, List<Integer> userShoppingCarCountList) {
		try {
			File file = new File(Common.REPORT_TEMPLATE);
			InputStream in = new FileInputStream(file);
			XSSFWorkbook wb = new XSSFWorkbook(in);
			XSSFSheet sheet = wb.getSheetAt(0);
			if (sheet != null) {
				XSSFCell cell = null;

				cell = sheet.getRow(0).getCell(0);
				cell.setCellValue("One Hour Report-" + year + "-" + month + "-" + day + " From " + hour + ":00 To " + hour + ":59");

				putData(userBuyCountList, sheet, 3);
				putData(userShoppingCarCountList, sheet, 7);

				FileOutputStream out = new FileOutputStream(Common.REPORT_ONE_HOUR);
				wb.write(out);
				out.close();
				System.err.println("done.");
			}
		} catch (Exception e) {
			System.err.println("Exception" + e);
		}
	}

	private static void putData(List<Integer> userBuyCountList, XSSFSheet sheet, int rowNum) {
		XSSFCell cell;
		if (userBuyCountList != null && userBuyCountList.size() > 0) {
			for (int i = 0; i < userBuyCountList.size(); i++) {
				cell = sheet.getRow(rowNum).getCell(i + 1);
				cell.setCellValue(userBuyCountList.get(i));
			}
		}
	}

	private static List<Integer> getData(HBaseDAO hBaseDAO, String year, String month, String day, String[] split, String preKey) {
		List<Integer> list = new ArrayList<Integer>();
		Result rs = hBaseDAO.getOneRowAndMultiColumn(Common.TABLE_USER_ACTION, preKey + "_" + year + "-" + month + "-" + day, split);
		for (Cell cell : rs.rawCells()) {
			String value = new String(CellUtil.cloneValue(cell)).split(":")[1].trim();
			value = value.substring(0, value.length() - 1);
			list.add(Integer.valueOf(value));
		}
		return list;
	}

	private static String[] generateQuailifers(String begin) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < 60;) {

			if (i == 0 || i == 5) {
				sb.append(begin).append("0").append(i).append("00").append(",");
			} else {
				sb.append(begin).append(i).append("00").append(",");
			}
			i = i + 5;
		}
		sb.append(begin).append("5959");
		String sbStr = sb.toString();
		String[] split = sbStr.split(",");
		return split;
	}
}

interface HBaseDAO {
	Result getOneRowAndMultiColumn(String tableName, String rowKey, String[] cols);
}

class HBaseDAOImpl implements HBaseDAO {

	HConnection hConnection = null;
	static Configuration conf = null;

	public HBaseDAOImpl() {
		conf = new Configuration();
		conf.set("hbase.zookeeper.quorum", Common.HBASE_ZOOKEEPER_LIST);
		try {
			hConnection = HConnectionManager.createConnection(conf);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public Result getOneRowAndMultiColumn(String tableName, String rowKey, String[] cols) {
		HTableInterface table = null;
		Result rsResult = null;
		try {
			table = hConnection.getTable(tableName);
			Get get = new Get(rowKey.getBytes());
			for (int i = 0; i < cols.length; i++) {
				get.addColumn(Common.COLUMN_FAMILY.getBytes(), cols[i].getBytes());
			}
			rsResult = table.get(get);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				table.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return rsResult;
	}

}

class Common {

	// report
	public static final String REPORT_TEMPLATE = "./resources/report.xlsx";
	public static final String REPORT_ONE_HOUR = "./resources/one_report.xlsx";

	public static final String DATE_FORMAT_YYYYDDMMHHMMSS = "yyyyMMddHHmmss";

	public static final String HBASE_ZOOKEEPER_LIST = "node1:2888,node2:2888,node3:2888";

	// key word
	public static final String KEY_WORD_BUY = "Buy";
	public static final String KEY_WORD_SHOPPING_CAR = "Shopping_Car";

	// hbase
	public static final String TABLE_USER_ACTION = "t_user_actions";
	public static final String COLUMN_FAMILY = "cf";

}